import logging

from spaceone.core.manager import BaseManager
from spaceone.config.model.user_config_model import UserConfig

_LOGGER = logging.getLogger(__name__)


class UserConfigManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user_config_model: UserConfig = self.locator.get_model('UserConfig')

    def create_user_config(self, params):
        def _rollback(user_config_vo):
            _LOGGER.info(f'[create_user_config._rollback] '
                         f'Delete config map : {user_config_vo.name}')
            user_config_vo.delete()

        user_config_vo: UserConfig = self.user_config_model.create(params)
        self.transaction.add_rollback(_rollback, user_config_vo)

        return user_config_vo

    def update_user_config(self, params):
        user_config_vo: UserConfig = self.get_user_config(params['name'], params['user_id'], params['domain_id'])
        return self.update_user_config_by_vo(params, user_config_vo)

    def update_user_config_by_vo(self, params, user_config_vo):
        def _rollback(old_data):
            _LOGGER.info(f'[update_user_config_by_vo._rollback] Revert Data : {old_data["name"]}')
            user_config_vo.update(old_data)

        self.transaction.add_rollback(_rollback, user_config_vo.to_dict())

        return user_config_vo.update(params)

    def delete_user_config(self, name, user_id, domain_id):
        user_config_vo: UserConfig = self.get_user_config(name, user_id, domain_id)
        user_config_vo.delete()

    def get_user_config(self, name, user_id, domain_id, only=None):
        return self.user_config_model.get(name=name, user_id=user_id, domain_id=domain_id, only=only)

    def filter_user_configs(self, **conditions):
        return self.user_config_model.filter(**conditions)

    def list_user_configs(self, query={}):
        return self.user_config_model.query(**query)

    def state_user_configs(self, query):
        return self.user_config_model.stat(**query)



import logging

from spaceone.core.service import *
from spaceone.core import utils
from spaceone.core.error import *
from spaceone.config.manager.user_config_manager import UserConfigManager

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class UserConfigService(BaseService):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user_config_mgr: UserConfigManager = self.locator.get_manager('UserConfigManager')

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['name', 'data', 'domain_id'])
    def create(self, params):
        """Create user config

        Args:
            params (dict): {
                'name': 'str',
                'data': 'dict',
                'tags': 'dict',
                'domain_id': 'str'
            }

        Returns:
            user_config_vo (object)
        """

        self._check_permission(params['domain_id'])

        params['user_id'] = self.transaction.get_meta('user_id')

        return self.user_config_mgr.create_user_config(params)

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['name', 'domain_id'])
    def update(self, params):
        """Update user config

        Args:
            params (dict): {
                'name': 'str',
                'data': 'dict',
                'tags': 'dict',
                'domain_id': 'str'
            }

        Returns:
            user_config_vo (object)
        """

        self._check_permission(params['domain_id'])

        params['user_id'] = self.transaction.get_meta('user_id')

        return self.user_config_mgr.update_user_config(params)

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['name', 'data', 'domain_id'])
    def set(self, params):
        """Set user config (create or update)

        Args:
            params (dict): {
                'name': 'str',
                'data': 'dict',
                'tags': 'dict',
                'domain_id': 'str'
            }

        Returns:
            user_config_vo (object)
        """

        self._check_permission(params['domain_id'])

        params['user_id'] = self.transaction.get_meta('user_id')

        user_type = self.transaction.get_meta('authorization.user_type')
        if user_type == 'DOMAIN_OWNER':
            raise ERROR_PERMISSION_DENIED()

        user_config_vos = self.user_config_mgr.filter_user_configs(domain_id=params['domain_id'],
                                                                   user_id=params['user_id'], name=params['name'])

        if user_config_vos.count() == 0:
            return self.user_config_mgr.create_user_config(params)
        else:
            return self.user_config_mgr.update_user_config_by_vo(params, user_config_vos[0])

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['name', 'domain_id'])
    def delete(self, params):
        """Delete user config

        Args:
            params (dict): {
                'name': 'str',
                'domain_id': 'str'
            }

        Returns:
            None
        """

        self._check_permission(params['domain_id'])
        user_id = self.transaction.get_meta('user_id')

        self.user_config_mgr.delete_user_config(params['name'], user_id, params['domain_id'])

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['name', 'domain_id'])
    def get(self, params):
        """Get user config

        Args:
            params (dict): {
                'name': 'str',
                'domain_id': 'str',
                'only': 'list'
            }

        Returns:
            user_config_vo (object)
        """

        user_id = self.transaction.get_meta('user_id')

        return self.user_config_mgr.get_user_config(params['name'], user_id, params['domain_id'], params.get('only'))

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['domain_id'])
    @append_query_filter(['name', 'user_id', 'domain_id'])
    @append_keyword_filter(['name'])
    def list(self, params):
        """ List user configs

        Args:
            params (dict): {
                'name': 'str',
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.Query)'
            }

        Returns:
            user_config_vos (objects)
            total_count (int)
        """

        query = params.get('query', {})
        return self.user_config_mgr.list_user_configs(query)

    @transaction(append_meta={'authorization.scope': 'USER'})
    @check_required(['query', 'domain_id'])
    @append_query_filter(['domain_id'])
    @append_keyword_filter(['name'])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)',
                'user_self': 'list', // from meta
            }

        Returns:
            values (list) : 'list of statistics data'

        """

        query = params.get('query', {})
        return self.user_config_mgr.state_user_configs(query)

    def _check_permission(self, request_domain_id):
        user_type = self.transaction.get_meta('authorization.user_type')
        user_domain_id = self.transaction.get_meta('domain_id')

        if user_type == 'DOMAIN_OWNER' or request_domain_id != user_domain_id:
            raise ERROR_PERMISSION_DENIED()



from spaceone.api.config.v1 import domain_config_pb2, domain_config_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class DomainConfig(BaseAPI, domain_config_pb2_grpc.DomainConfigServicer):

    pb2 = domain_config_pb2
    pb2_grpc = domain_config_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainConfigService', metadata) as domain_config_service:
            return self.locator.get_info('DomainConfigInfo', domain_config_service.create(params))

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainConfigService', metadata) as domain_config_service:
            return self.locator.get_info('DomainConfigInfo', domain_config_service.update(params))

    def set(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainConfigService', metadata) as domain_config_service:
            return self.locator.get_info('DomainConfigInfo', domain_config_service.set(params))

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainConfigService', metadata) as domain_config_service:
            domain_config_service.delete(params)
            return self.locator.get_info('EmptyInfo')

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainConfigService', metadata) as domain_config_service:
            return self.locator.get_info('DomainConfigInfo', domain_config_service.get(params))

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainConfigService', metadata) as domain_config_service:
            domain_config_vos, total_count = domain_config_service.list(params)
            return self.locator.get_info('DomainConfigsInfo', domain_config_vos,
                                         total_count, minimal=self.get_minimal(params))

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('DomainConfigService', metadata) as domain_config_service:
            return self.locator.get_info('StatisticsInfo', domain_config_service.stat(params))



DATABASE_AUTO_CREATE_INDEX = True
DATABASES = {
    'default': {
        'db': 'config',
        'host': 'localhost',
        'port': 27017,
        'username': '',
        'password': ''
    }
}

CACHES = {
    'default': {},
    'local': {
        'backend': 'spaceone.core.cache.local_cache.LocalCache',
        'max_size': 128,
        'ttl': 300
    }
}

HANDLERS = {
}



import functools
from spaceone.api.config.v1 import domain_config_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.config.model.domain_config_model import DomainConfig

__all__ = ['DomainConfigInfo', 'DomainConfigsInfo']


def DomainConfigInfo(domain_config_vo: DomainConfig, minimal=False):
    info = {
        'name': domain_config_vo.name,
    }

    if not minimal:
        info.update({
            'data': change_struct_type(domain_config_vo.data),
            'tags': change_struct_type(domain_config_vo.tags),
            'domain_id': domain_config_vo.domain_id,
            'updated_at': utils.datetime_to_iso8601(domain_config_vo.updated_at),
            'created_at': utils.datetime_to_iso8601(domain_config_vo.created_at)
        })

    return domain_config_pb2.DomainConfigInfo(**info)


def DomainConfigsInfo(domain_config_vos, total_count, **kwargs):
    return domain_config_pb2.DomainConfigsInfo(results=list(
        map(functools.partial(DomainConfigInfo, **kwargs), domain_config_vos)), total_count=total_count)



from spaceone.api.config.v1 import user_config_pb2, user_config_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class UserConfig(BaseAPI, user_config_pb2_grpc.UserConfigServicer):

    pb2 = user_config_pb2
    pb2_grpc = user_config_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('UserConfigService', metadata) as user_config_service:
            return self.locator.get_info('UserConfigInfo', user_config_service.create(params))

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('UserConfigService', metadata) as user_config_service:
            return self.locator.get_info('UserConfigInfo', user_config_service.update(params))

    def set(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('UserConfigService', metadata) as user_config_service:
            return self.locator.get_info('UserConfigInfo', user_config_service.set(params))

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('UserConfigService', metadata) as user_config_service:
            user_config_service.delete(params)
            return self.locator.get_info('EmptyInfo')

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('UserConfigService', metadata) as user_config_service:
            return self.locator.get_info('UserConfigInfo', user_config_service.get(params))

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('UserConfigService', metadata) as user_config_service:
            user_config_vos, total_count = user_config_service.list(params)
            return self.locator.get_info('UserConfigsInfo', user_config_vos,
                                         total_count, minimal=self.get_minimal(params))

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('UserConfigService', metadata) as user_config_service:
            return self.locator.get_info('StatisticsInfo', user_config_service.stat(params))



from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel


class DomainConfigTag(EmbeddedDocument):
    key = StringField(max_length=255)
    value = StringField(max_length=255)


class DomainConfig(MongoModel):
    name = StringField(max_length=255, unique_with='domain_id')
    data = DictField()
    tags = DictField()
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)
    updated_at = DateTimeField(auto_now=True)

    meta = {
        'updatable_fields': [
            'name',
            'data',
            'tags',
            'updated_at'
        ],
        'minimal_fields': [
            'name'
        ],
        'ordering': [
            'name'
        ],
        'indexes': [
            'name',
            'domain_id',
        ]
    }



import logging

from spaceone.core.service import *
from spaceone.core import utils
from spaceone.config.manager.domain_config_manager import DomainConfigManager

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class DomainConfigService(BaseService):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.domain_config_mgr: DomainConfigManager = self.locator.get_manager('DomainConfigManager')

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['name', 'data', 'domain_id'])
    def create(self, params):
        """Create domain config

        Args:
            params (dict): {
                'name': 'str',
                'data': 'dict',
                'tags': 'dict',
                'domain_id': 'str'
            }

        Returns:
            domain_config_vo (object)
        """

        return self.domain_config_mgr.create_domain_config(params)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['name', 'domain_id'])
    def update(self, params):
        """Update domain config

        Args:
            params (dict): {
                'name': 'str',
                'data': 'dict',
                'tags': 'dict',
                'domain_id': 'str'
            }

        Returns:
            domain_config_vo (object)
        """

        return self.domain_config_mgr.update_domain_config(params)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['name', 'data', 'domain_id'])
    def set(self, params):
        """Set domain config (create or update)

        Args:
            params (dict): {
                'name': 'str',
                'data': 'dict',
                'tags': 'dict',
                'domain_id': 'str'
            }

        Returns:
            domain_config_vo (object)
        """

        domain_id = params['domain_id']

        domain_config_vos = self.domain_config_mgr.filter_domain_configs(domain_id=domain_id, name=params['name'])

        if domain_config_vos.count() == 0:
            return self.domain_config_mgr.create_domain_config(params)
        else:
            return self.domain_config_mgr.update_domain_config_by_vo(params, domain_config_vos[0])

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['name', 'domain_id'])
    def delete(self, params):
        """Delete domain config

        Args:
            params (dict): {
                'name': 'str',
                'domain_id': 'str'
            }

        Returns:
            None
        """

        self.domain_config_mgr.delete_domain_config(params['name'], params['domain_id'])

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['name', 'domain_id'])
    def get(self, params):
        """Get domain config

        Args:
            params (dict): {
                'name': 'str',
                'domain_id': 'str',
                'only': 'list'
            }

        Returns:
            domain_config_vo (object)
        """

        return self.domain_config_mgr.get_domain_config(params['name'], params['domain_id'], params.get('only'))

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['domain_id'])
    @append_query_filter(['name', 'domain_id'])
    @append_keyword_filter(['name'])
    def list(self, params):
        """ List domain configs

        Args:
            params (dict): {
                'name': 'str',
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.Query)'
            }

        Returns:
            domain_config_vos (objects)
            total_count (int)
        """

        query = params.get('query', {})
        return self.domain_config_mgr.list_domain_configs(query)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['query', 'domain_id'])
    @append_query_filter(['domain_id'])
    @append_keyword_filter(['name'])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)'
            }

        Returns:
            values (list) : 'list of statistics data'

        """

        query = params.get('query', {})
        return self.domain_config_mgr.state_domain_configs(query)



import functools
from spaceone.api.config.v1 import user_config_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.config.model.user_config_model import UserConfig

__all__ = ['UserConfigInfo', 'UserConfigsInfo']


def UserConfigInfo(user_config_vo: UserConfig, minimal=False):
    info = {
        'name': user_config_vo.name,
    }

    if not minimal:
        info.update({
            'data': change_struct_type(user_config_vo.data),
            'tags': change_struct_type(user_config_vo.tags),
            'user_id': user_config_vo.user_id,
            'domain_id': user_config_vo.domain_id,
            'updated_at': utils.datetime_to_iso8601(user_config_vo.updated_at),
            'created_at': utils.datetime_to_iso8601(user_config_vo.created_at)
        })

    return user_config_pb2.UserConfigInfo(**info)


def UserConfigsInfo(user_config_vos, total_count, **kwargs):
    return user_config_pb2.UserConfigsInfo(results=list(
        map(functools.partial(UserConfigInfo, **kwargs), user_config_vos)), total_count=total_count)



import logging

from spaceone.core.manager import BaseManager
from spaceone.config.model.domain_config_model import DomainConfig

_LOGGER = logging.getLogger(__name__)


class DomainConfigManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.domain_config_model: DomainConfig = self.locator.get_model('DomainConfig')

    def create_domain_config(self, params):
        def _rollback(domain_config_vo):
            _LOGGER.info(f'[create_domain_config._rollback] '
                         f'Delete domain config : {domain_config_vo.name}')
            domain_config_vo.delete()

        domain_config_vo: DomainConfig = self.domain_config_model.create(params)
        self.transaction.add_rollback(_rollback, domain_config_vo)

        return domain_config_vo

    def update_domain_config(self, params):
        domain_config_vo: DomainConfig = self.get_domain_config(params['name'], params['domain_id'])
        return self.update_domain_config_by_vo(params, domain_config_vo)

    def update_domain_config_by_vo(self, params, domain_config_vo):
        def _rollback(old_data):
            _LOGGER.info(f'[update_domain_config_by_vo._rollback] Revert Data : {old_data["name"]}')
            domain_config_vo.update(old_data)

        self.transaction.add_rollback(_rollback, domain_config_vo.to_dict())

        return domain_config_vo.update(params)

    def delete_domain_config(self, name, domain_id):
        domain_config_vo: DomainConfig = self.get_domain_config(name, domain_id)
        domain_config_vo.delete()

    def get_domain_config(self, name, domain_id, only=None):
        return self.domain_config_model.get(name=name, domain_id=domain_id, only=only)

    def filter_domain_configs(self, **conditions):
        return self.domain_config_model.filter(**conditions)

    def list_domain_configs(self, query={}):
        return self.domain_config_model.query(**query)

    def state_domain_configs(self, query):
        return self.domain_config_model.stat(**query)
